@@ -799,7 +799,7 @@ int mutt_index_menu (void)
 	  else
 	    menu->current = 0;
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  if (Sort & SORT_THREADS)
+	  if ((Sort & SORT_MASK) == SORT_THREADS)
 	    mutt_linearize_tree (Context, 0);
 	}
 	break;	  
