@@ -888,7 +888,9 @@ void text_enriched_handler (BODY *a, STATE *s)
 
 /*
  * An implementation of RFC 2646.
- * 
+ *
+ * NOTE: This still has to be made UTF-8 aware.
+ *
  */
 
 #define FLOWED_MAX 77
@@ -926,8 +928,8 @@ static void flowed_stuff (STATE *s, char *cont, int level)
      * some text on the line which looks like it's quoted, turn off 
      * ANSI colors, so quote coloring doesn't affect this line. 
      */
-    if (*cont && !level && flowed_maybe_quoted (cont))
-      state_puts ("\033[0;m",s);
+    if (*cont && !level && !mutt_strcmp (Pager, "builtin") && flowed_maybe_quoted (cont))
+      state_puts ("\033[0m",s);
   }
   else if ((*cont == ' ') || (*cont == '>') || (!level && !mutt_strncmp (cont, "From ", 5)))
     state_putc (' ', s);
@@ -941,6 +943,20 @@ static char *flowed_skip_indent (char *prefix, char *cont)
   return cont;
 }
 
+static int flowed_visual_strlen (char *l, int i)
+{
+  int j;
+  for (j = 0; *l; l++)
+  {
+    if (*l == '\t')
+      j += 8 - ((i + j) % 8);
+    else
+      j++;
+  }
+  
+  return j;
+}
+
 static void text_plain_flowed_handler (BODY *a, STATE *s)
 {
   char line[LONG_STRING];
@@ -950,12 +966,12 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
   int  last_quoted;
   int  full = 1;
   int  last_full;
-  int  col = 0;
+  int  col = 0, tmpcol;
 
+  int  i_add = 0;
   int  add = 0;
   int  soft = 0;
-  int  max;
-  int  l;
+  int  l, rl;
   
   int  flowed_max;
   int  bytes = a->length;
@@ -965,7 +981,7 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
   char *tail = NULL;
   char *lc = NULL;
   char *t;
-
+  
   *indent = '\0';
   
   if (s->prefix)
@@ -1039,7 +1055,8 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
 	cont++;
 
       /* If there is an indentation, record it. */
-      cont = flowed_skip_indent (indent, cont);
+      cont  = flowed_skip_indent (indent, cont);
+      i_add = flowed_visual_strlen (indent, quoted + add);
     }
     else
     {
@@ -1071,23 +1088,23 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
       /* try to find a point for word wrapping */
 
     retry_wrap:
-      l = mutt_strlen (cont);
-      if (quoted + add + col + l > flowed_max)
+      l  = flowed_visual_strlen (cont, quoted + i_add + add + col);
+      rl = mutt_strlen (cont);
+      if (quoted + i_add + add + col + l > flowed_max)
       {
 	actually_wrap = 1;
-	if ((max = flowed_max - quoted - add - col)
-	    > l)
-	  max = l;
 
-	for (t = cont + max; t > cont; t--)
+	for (tmpcol = quoted + i_add + add + col, t = cont;
+	     *t && tmpcol < flowed_max; t++)
 	{
 	  if (*t == ' ' || *t == '\t')
-	  {
 	    tail = t;
-	    break;
-	  }
+	  if (*t == '\t')
+	    tmpcol = (tmpcol & ~7) + 8;
+	  else
+	    tmpcol++;
 	}
-
+	
 	if (tail)
 	{
 	  *tail++ = '\0';
@@ -1096,7 +1113,7 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
       }
 
       /* We seem to be desperate.  Get me a new line, and retry. */
-      if (!tail && (quoted + add + col + mutt_strlen (cont) > flowed_max) && col)
+      if (!tail && (quoted + add + col + i_add + l > flowed_max) && col)
       {
 	state_putc ('\n', s);
 	col = 0;
@@ -1133,7 +1150,7 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
 
       /* output the text */
       state_puts (cont, s);
-      col += strlen (cont);
+      col += flowed_visual_strlen (cont, quoted + i_add + add + col);
       
       /* possibly indicate a soft line break */
       if (soft == 2)
@@ -1299,6 +1316,7 @@ static void alternative_handler (BODY *a, STATE *s)
   else if (s->flags & M_DISPLAY)
   {
     /* didn't find anything that we could display! */
+    state_mark_attach (s);
     state_puts(_("[-- Error:  Could not display any parts of Multipart/Alternative! --]\n"), s);
   }
 }
@@ -1416,6 +1434,7 @@ void multipart_handler (BODY *a, STATE *s)
   {
     if (s->flags & M_DISPLAY)
     {
+      state_mark_attach (s);
       state_printf (s, _("[-- Attachment #%d"), count);
       if (p->description || p->filename || p->form_name)
       {
@@ -1427,6 +1446,7 @@ void multipart_handler (BODY *a, STATE *s)
 
       mutt_pretty_size (length, sizeof (length), p->length);
       
+      state_mark_attach (s);
       state_printf (s, _("[-- Type: %s/%s, Encoding: %s, Size: %s --]\n"),
 		    TYPE (p), p->subtype, ENCODING (p->encoding), length);
       if (!option (OPTWEED))
@@ -1486,6 +1506,7 @@ void autoview_handler (BODY *a, STATE *s)
 
     if (s->flags & M_DISPLAY)
     {
+      state_mark_attach (s);
       state_printf (s, _("[-- Autoview using %s --]\n"), command);
       mutt_message(_("Invoking autoview command: %s"),command);
     }
@@ -1517,7 +1538,10 @@ void autoview_handler (BODY *a, STATE *s)
     {
       mutt_perror _("Can't create filter");
       if (s->flags & M_DISPLAY)
+      {
+	state_mark_attach (s);
 	state_printf (s, _("[-- Can't run %s. --]\n"), command);
+      }
       goto bail;
     }
     
@@ -1531,8 +1555,11 @@ void autoview_handler (BODY *a, STATE *s)
       /* check for data on stderr */
       if (fgets (buffer, sizeof(buffer), fperr)) 
       {
-	if (s->flags & M_DISPLAY) 
+	if (s->flags & M_DISPLAY)
+	{
+	  state_mark_attach (s);
 	  state_printf (s, _("[-- Autoview stderr of %s --]\n"), command);
+	}
 
 	state_puts (s->prefix, s);
 	state_puts (buffer, s);
@@ -1550,8 +1577,11 @@ void autoview_handler (BODY *a, STATE *s)
       if (fgets (buffer, sizeof(buffer), fperr))
       {
 	if (s->flags & M_DISPLAY)
+	{
+	  state_mark_attach (s);
 	  state_printf (s, _("[-- Autoview stderr of %s --]\n"), 
 			command);
+	}
 	
 	state_puts (buffer, s);
 	mutt_copy_stream (fperr, s->fpout);
@@ -1584,7 +1614,10 @@ static void external_body_handler (BODY *b, STATE *s)
   if (!access_type)
   {
     if (s->flags & M_DISPLAY)
+    {
+      state_mark_attach (s);
       state_puts (_("[-- Error: message/external-body has no access-type parameter --]\n"), s);
+    }
     return;
   }
 
@@ -1600,7 +1633,8 @@ static void external_body_handler (BODY *b, STATE *s)
     {
       char *length;
       char pretty_size[10];
-
+      
+      state_mark_attach (s);
       state_printf (s, _("[-- This %s/%s attachment "),
 	       TYPE(b->parts), b->parts->subtype);
       length = mutt_get_parameter ("length", b->parameter);
@@ -1613,9 +1647,15 @@ static void external_body_handler (BODY *b, STATE *s)
       state_puts (_("has been deleted --]\n"), s);
 
       if (expire != -1)
+      {
+	state_mark_attach (s);
 	state_printf (s, _("[-- on %s --]\n"), expiration);
+      }
       if (b->parts->filename)
+      {
+	state_mark_attach (s);
 	state_printf (s, _("[-- name: %s --]\n"), b->parts->filename);
+      }
 
       mutt_copy_hdr (s->fpin, s->fpout, ftell (s->fpin), b->parts->offset,
 		     (option (OPTWEED) ? (CH_WEED | CH_REORDER) : 0) |
@@ -1626,10 +1666,12 @@ static void external_body_handler (BODY *b, STATE *s)
   {
     if (s->flags & M_DISPLAY)
     {
-      state_printf (s, _("[-- This %s/%s attachment is not included, --]\n"
-			 "[-- and the indicated external source has --]\n"
-			 "[-- expired. --]\n"),
+      state_mark_attach (s);
+      state_printf (s, _("[-- This %s/%s attachment is not included, --]\n"),
 		    TYPE(b->parts), b->parts->subtype);
+      state_attach_puts (_("[-- and the indicated external source has --]\n"), s);
+      state_attach_puts (_("[-- expired. --]\n"), s);
+
       mutt_copy_hdr(s->fpin, s->fpout, ftell (s->fpin), b->parts->offset,
 		    (option (OPTWEED) ? (CH_WEED | CH_REORDER) : 0) |
 		    CH_DECODE, NULL);
@@ -1639,10 +1681,14 @@ static void external_body_handler (BODY *b, STATE *s)
   {
     if (s->flags & M_DISPLAY)
     {
+      state_mark_attach (s);
       state_printf (s,
-	       _("[-- This %s/%s attachment is not included, --]\n"
-		 "[-- and the indicated access-type %s is unsupported --]\n"),
-	       TYPE(b->parts), b->parts->subtype, access_type);
+		    _("[-- This %s/%s attachment is not included, --]\n"),
+		    TYPE (b->parts), b->parts->subtype);
+      state_mark_attach (s);
+      state_printf (s, 
+		    _("[-- and the indicated access-type %s is unsupported --]\n"),
+		    access_type);
       mutt_copy_hdr (s->fpin, s->fpout, ftell (s->fpin), b->parts->offset,
 		     (option (OPTWEED) ? (CH_WEED | CH_REORDER) : 0) |
 		     CH_DECODE , NULL);
@@ -1877,6 +1923,7 @@ void mutt_body_handler (BODY *b, STATE *s)
   }
   else if (s->flags & M_DISPLAY)
   {
+    state_mark_attach (s);
     state_printf (s, _("[-- %s/%s is unsupported "), TYPE (b), b->subtype);
     if (!option (OPTVIEWATTACH))
     {
