@@ -57,6 +57,7 @@
 #define R_PAGER		(1<<1)
 #define R_RESORT	(1<<2)	/* resort the mailbox */
 #define R_RESORT_SUB	(1<<3)	/* resort subthreads */
+#define R_RESORT_INIT	(1<<4)  /* resort from scratch */
 #define R_BOTH		(R_INDEX | R_PAGER)
 #define R_RESORT_BOTH	(R_RESORT | R_RESORT_SUB)
 
@@ -147,6 +148,17 @@ struct option_t MuttVars[] = {
   ** Controls whether 8-bit data is converted to 7-bit using either Quoted-
   ** Printable or Base64 encoding when sending mail.
   */
+  { "allow_ansi",      DT_BOOL, R_NONE, OPTALLOWANSI, 0 },
+  /*
+  ** .pp
+  ** Controls whether ANSI color codes in messages (and color tags in 
+  ** rich text messages) are to be interpreted.
+  ** Messages containing these codes are rare, but if this option is set,
+  ** their text will be colored accordingly. Note that this may override
+  ** your color choices, and even present a security problem, since a
+  ** message could include a line like "[-- PGP output follows ..." and
+  ** give it the same color as your attachment color.
+  */
   { "alternates",	DT_RX,	 R_BOTH, UL &Alternates, 0 },
   /*
   ** .pp
@@ -426,6 +438,14 @@ struct option_t MuttVars[] = {
   ** .pp
   ** Example: set dsn_return=hdrs
   */
+  { "duplicate_threads",	DT_BOOL, R_RESORT|R_RESORT_INIT|R_INDEX, OPTDUPTHREADS, 1 },
+  /*
+  ** .pp
+  ** This variable controls whether mutt, when sorting by threads, threads
+  ** messages with the same message-id together.  If it is set, it will indicate
+  ** that it thinks they are duplicates of each other with an equals sign
+  ** in the thread diagram.
+  */
   { "edit_headers",	DT_BOOL, R_NONE, OPTEDITHDRS, 0 },
   /*
   ** .pp
@@ -640,7 +660,7 @@ struct option_t MuttVars[] = {
   ** affect the generation of Message-IDs, and it will not lead to the 
   ** cut-off of first-level domains.
   */
-  { "hide_missing",	DT_BOOL, R_NONE, OPTHIDEMISSING, 1 },
+  { "hide_missing",	DT_BOOL, R_RESORT|R_INDEX, OPTHIDEMISSING, 1 },
   /*
   ** .pp
   ** When set, mutt will not indicate the presence of missing messages
@@ -2035,7 +2055,7 @@ struct option_t MuttVars[] = {
   ** You may optionally use the reverse- prefix to specify reverse sorting
   ** order (example: set sort_browser=reverse-date).
   */
-  { "sort_re",		DT_BOOL, R_INDEX|R_RESORT_BOTH, OPTSORTRE, 1 },
+  { "sort_re",		DT_BOOL, R_INDEX|R_RESORT|R_RESORT_INIT, OPTSORTRE, 1 },
   /*
   ** .pp
   ** This variable is only useful when sorting by threads with
@@ -2140,7 +2160,7 @@ struct option_t MuttVars[] = {
   ** Setting this variable causes the ``status bar'' to be displayed on
   ** the first line of the screen rather than near the bottom.
   */
-  { "strict_threads",	DT_BOOL, R_RESORT|R_INDEX, OPTSTRICTTHREADS, 0 },
+  { "strict_threads",	DT_BOOL, R_RESORT|R_RESORT_INIT|R_INDEX, OPTSTRICTTHREADS, 0 },
   /*
   ** .pp
   ** If set, threading will only make use of the ``In-Reply-To'' and
@@ -2167,7 +2187,7 @@ struct option_t MuttVars[] = {
   ** .pp
   ** Note that $$indent_string is ignored when this option is set.
   */
-  { "thread_received",	DT_BOOL, R_NONE, OPTTHREADRECEIVED, 0 },
+  { "thread_received",	DT_BOOL, R_RESORT|R_RESORT_INIT|R_INDEX, OPTTHREADRECEIVED, 0 },
   /*
   ** .pp
   ** When set, mutt uses the date received rather than the date sent
