@@ -452,7 +452,7 @@ static int mix_chain_add (MIXCHAIN *chain, const char *s,
   if (chain->cl >= MAXMIXES)
     return -1;
   
-  if (!mutt_strcmp (s, "0") || !mutt_strcasecmp (s, "<random>"))
+  if (!mutt_strcmp (s, "0") || !ascii_strcasecmp (s, "<random>"))
   {
     chain->ch[chain->cl++] = 0;
     return 0;
@@ -460,7 +460,7 @@ static int mix_chain_add (MIXCHAIN *chain, const char *s,
 
   for (i = 0; type2_list[i]; i++)
   {
-    if (!mutt_strcasecmp (s, type2_list[i]->shortname))
+    if (!ascii_strcasecmp (s, type2_list[i]->shortname))
     {
       chain->ch[chain->cl++] = i;
       return 0;
