@@ -549,7 +549,7 @@ LIST *mutt_make_references(ENVELOPE *e)
   else
     l = mutt_copy_list (e->in_reply_to);
   
-  if(e->message_id)
+  if (e->message_id)
   {
     t = mutt_new_list();
     t->data = safe_strdup(e->message_id);
@@ -615,13 +615,13 @@ mutt_make_reference_headers (ENVELOPE *curenv, ENVELOPE *env, CONTEXT *ctx)
     
     for(i = 0; i < ctx->vcount; i++)
     {
-      while(*p) p = &(*p)->next;
+      while (*p) p = &(*p)->next;
       while (*q) q = &(*q)->next;
       
       h = ctx->hdrs[ctx->v2r[i]];
-      if(h->tagged)
+      if (h->tagged)
       {
-	*p = mutt_make_references(h->env);
+	*p = mutt_make_references (h->env);
 	if (h->env->message_id)
 	{
 	  *q = mutt_new_list ();
@@ -1276,22 +1276,8 @@ ci_send_message (int flags,		/* send mode */
   else if (! (flags & SENDBATCH))
   {
     struct stat st;
-    time_t mtime;
-    struct utimbuf utim;
-
-    stat (msg->content->filename, &st);
-    mtime = st.st_mtime;
-    if (mtime == time (NULL))
-    {
-      /* Decrease the file's modification time by 1 second so we are sure
-       * to find out if the `editor' program changes it in less than 1 second.
-       */
-      mtime -= 1;
-      utim.actime = mtime;
-      utim.modtime = mtime;
-      utime (msg->content->filename, &utim);
-    }
-
+    time_t mtime = mutt_decrease_mtime (msg->content->filename, NULL);
+    
     mutt_update_encoding (msg->content);
 
     /* If the this isn't a text message, look for a mailcap edit command */
